const BaseStrategyTrader = global.BaseStrategyTrader || require('./lib/trader/BaseStrategyTrader')
const CCXTTrader = global.CCXTTrader || require('./lib/trader/CCXTTrader')

module.exports = class extends BaseStrategyTrader(CCXTTrader) {
  static async start ({ apiToken } = {}) {
    const Strategy = this
    let params = Strategy.options || Strategy.defaultOptions
    params.apiToken = apiToken
    let _strategy = new Strategy(params)
    await _strategy._onInit()
    await _strategy._onTick()
    await _strategy._onExit()
  }

  static get defaultOptions () {
    const defaultSymbol = 'BTC/USDT'
    return {
      defaultSymbol,
      exchanges: {
        jsmex: {
          apiToken: process.env.API_TOKEN,
          wwwDomain: 'jsmex.io',
          apiDomain: 'api.jsmex.io'
        }
      },
      ccxt: {
        fetchDepth: {
          symbol: defaultSymbol // overridden by defaultSymbol
        },
        fetchOHLCV: {
          symbol: defaultSymbol, // overridden by defaultSymbol
          resolution: '10',
          since: Date.now(),
          limit: 25
        },
        loadMarkets: {},
        cancelOrder: {
          symbol: defaultSymbol // overridden by defaultSymbol
          // id,    {override}
        },
        fetchOrders: {
          symbol: defaultSymbol // overridden by defaultSymbol
        },
        fetchTransactions: {
          symbol: defaultSymbol, // overridden by defaultSymbol
          since: Date.now() / 1000,
          limit: 25
        },
        putOrder: {
          symbol: defaultSymbol // overridden by defaultSymbol
          // type,    {override}
          // side,    {override}
          // amount,  {override}
          // price    {override}
        },
        pipeline: {
          symbol: defaultSymbol // overridden by defaultSymbol
          // pipeline {pverride}
        },
        balance: {}
      }
    }
  }
}

module.exports.classes = {
  BaseStrategyTrader: BaseStrategyTrader,
  CCXTTrader: CCXTTrader,
  ITrader: global.ITrader || require('./lib/trader/ITrader'),
  CCXTExchange: global.CCXTExchange || require('./lib/exchange/CCXTExchange'),
  IExchange: global.IExchange || require('./lib/exchange/IExchange'),
  data: require('./lib/data'),
  Symbol: require('./lib/utils/Symbol'),
  UID: require('./lib/utils/UID')
}
