const IDTO = global.IDTO || require('./IDTO')
const Joi = global.Joi || require('../../static_dependencies/joi/joi-browser.min.js')

/**
 * DepthDTO class
 *
 * @author jsb4ch <jsb4ch@jsmex.io>
 * @since 1.0.0
*/
class DepthDTO extends IDTO {
  constructor (uid) {
    super(uid)

    this.symbolId = ''
    this.depthId = `${this.symbolId}-depth`

    this.createdAt = new Date()
    this.asks = []
    this.bids = []
    this.validate()
  }

  /**
   * DepthDTO get symbol
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.69
  */
  get symbol () {
    return this.symbolId
  }

  /**
   * DepthDTO set symbol
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.69
  */
  set symbol (symbolId) {
    this.symbolId = symbolId
  }

  /**
   * DepthDTO validation method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.0
  */
  validate () {
    this.depthId = `${this.symbolId}-depth`
    let data = {
      symbolId: this.symbolId,
      depthId: this.depthId,
      createdAt: this.createdAt,
      asks: this.asks,
      bids: this.bids
    }
    let joiValidDepth = Joi.object()
      .keys({
        symbolId: Joi.alternatives().try(Joi.string(), Joi.string().valid('')).required(),
        depthId: Joi.string().required(),
        createdAt: Joi.any().required(),
        asks: Joi.array()
          .items(
            Joi.array()
              .items(
                Joi.string(),
                Joi.number()
              )
              .length(2)
          )
          .required(),
        bids: Joi.array()
          .items(
            Joi.array()
              .items(
                Joi.string(),
                Joi.number()
              )
              .length(2)
          )
          .required()
      })

    let result = Joi.validate(data, joiValidDepth)
    if (result.error) {
      throw result.error
    }
    return result.value
  }

  /**
   * DepthDTO toObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.14
   * @returns {Object}
  */
  toObject () {
    return {
      uid: this.uid,
      symbolId: this.symbolId,
      depthId: this.depthId,
      createdAt: this.createdAt,
      asks: this.asks,
      bids: this.bids
    }
  }

  /**
   * DepthDTO fromObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.14
   * @param objectData object
   * @returns DepthDTO
  */
  static fromObject (objectData) {
    let depthDTO = new DepthDTO(objectData.uid)
    depthDTO.depthId = objectData.depthId
    depthDTO.symbolId = objectData.symbolId
    depthDTO.createdAt = new Date(objectData.createdAt)
    depthDTO.asks = objectData.asks
    depthDTO.bids = objectData.bids
    depthDTO.validate()
    return depthDTO
  }
}

module.exports = DepthDTO
